@@ -47,8 +47,10 @@ HISTORY
   2011-01-20 V1.2 PB only info in case of an error for zlib version<1.2.4
   2012-06-24 V1.3 PB cmpr_inflatefile and cmpr_deflatefile added
   2012-06-26 V1.4 PB CMPRUTILS added
+  2012-12-29 V1.5 PB zlib defines z_strm.avail_in as uInt =>
+                     print format changed from %lu to %u
 --------------------------------------------------------------------------*/
-# define CMPR_VERSION "cmpr : V1.4 Peter Boesecke 2012-06-26 -- zlib : " ZLIB_VERSION
+# define CMPR_VERSION "cmpr : V1.5 Peter Boesecke 2012-12-29 -- zlib : " ZLIB_VERSION
 /****************************************************************************
 *  Include                                                                  *
 ****************************************************************************/
@@ -1007,7 +1009,7 @@ int cmpr_deflatefile ( FILE * outp, FILE * inp,
       if (strm.avail_in!=0) {
         /* all input must be used */
         if (CMPR_debug) {
-          printf(" cmpr_deflatefile: not all input deflated (%lu bytes unprocessed)\n",
+          printf(" cmpr_deflatefile: not all input deflated (%u bytes unprocessed)\n",
             strm.avail_in);
         }
         errval=Z_ERRNO;
