@@ -37,6 +37,7 @@ HISTORY
   2008-12-05  PB V1.1 open output text file as binary file to avoid
                       doubling of \r (\r\r\n) at the end of the line when
                       compiled on windows.
+  2012-08-30  PB V1.2 BMode: FloatIEEE64 and FloatIEEE128, VAX types removed
 
 ***************************************************************************/
 # include <stdlib.h>
@@ -84,8 +85,7 @@ PRIVATE char end_of_line[3] = { '\r', '\n', '\0' };
 enum BMode { TextMode,
              Unsigned8=1, Signed8,      Unsigned16,    Signed16,
              Unsigned32,  Signed32,     Unsigned64,    Signed64,
-             FloatIEEE32, DoubleIEEE64, Unused11,      Unused12,
-             FloatVAX32,  DoubleVAX64,  FloatConvex32, DoubleConvex64,
+             FloatIEEE32, FloatIEEE64,  FloatIEEE128,  Unused12,
              EndBMode };
 
 /***************************************************************************
