@@ -60,10 +60,19 @@
   A "=" in the argument list chooses the default.
 
 2 HISTORY
-  2012-02-09  PB 1.0
+  2012-02-09  PB 4.00
+  2012-06-26  PB 4.01 strlib.h included, 
+                       4) calculate variance of the average
+                         --> variance calculation corrected
+  2012-06-27  PB 4.02 percentchar added to avoid lcc format string bug
+  2012-09-12  PB 4.03 pq splitted into loq, upq with defaults loq=0.1, upq=0.9
+  2012-09-12  PB 4.04 average: 
+                      amode: average value corrected in variance calculation,
+                             quantil is now pq-quantil of the option pq
+                      fmode: quantil filters between lpq and upq.
 
 ---*/
-# define Version  "saxs_average V4.00 2012-02-09, Peter Boesecke"
+# define Version  "saxs_average V4.04 2012-09-13, Peter Boesecke"
 
 # include <errno.h>
 # include <stdio.h>
@@ -71,6 +80,7 @@
 # include <fcntl.h>
 
 # include "statfunc.h"
+# include "strlib.h"
 # include "SaxsPrograms.h"
 
 # define FREE( a) { if ( a) { free( a); a = NULL; } }
@@ -78,11 +88,23 @@
 # define Usage "[options] \n\
                 <i1nam> <onam> <i1fst> <i1lst> <i1inc> \n\
                 <odum> <odim1> <odim2> <ofac> <ocon> \n\n\
-                Calculation of the average and the variance of a series\n\
-                of images. All images that are to average will be kept\n\
-                in memory. If the option +var is set also the variance with \n\
-                respect to the calculated average is calculated and written \n\
-                into the error image (-omem -1).\n\
+                Calculation of the average and the variance of a series of\n\
+                images. All images that will be averaged are first read\n\
+                and kept in memory. If the option +var is set (default) also\n\
+                the variance with respect to the calculated average is \n\
+                calculated and written into the error image (-omem -1).\n\
+                The variance is only well defined in averaging mode mean\n\
+                which is the default.\n\
+                Flatfield:\n\
+                To remove cosmics when averaging a series of images the\n\
+                filter mode median can be used.\n\
+                Hot pixel mask:\n\
+                To create a hot pixel mask a series of dark images should\n\
+                be averaged with averaging mode median or quantil. Only\n\
+                pixel values that are in more than 50% of the cases hot\n\
+                (median) will get the hot value, all others will have \n\
+                the usual dark value. The averaging mode quantil allows\n\
+                to choose other levels than 50%.\n\
                 options: -add <number of images to average>\n\
                               (default 0 to average all)\n\
                          -amod   <mean|median|quantil> average function\n\
@@ -97,8 +119,18 @@
                                    (default 5)\n\
                          -mins   <minimum sigma> for filter\n\
                                    (default 0.5)\n\
-                         -pq     <quantil value p> [1-p..1] (0.5<=p<1)\n\
-                                   (default 0.9)"
+                         -loq    <lower quantil (fmode)> [0..1] (0<=loq<1)\n\
+                                   (default 0.1)\n\
+                         -upq    <upper quantil (fmode)> [0..1] (0<=upq<1)\n\
+                                   (default 0.1)\n\
+                         -pq     <quantil (amode)> [0..1] (0<=pq<=1)\n\
+                                   (default 0.5 == median calculation)\n\
+                In averaging mode quantil the pq-quantil is returned. For\n\
+                pq==0.5 (default) the median is calculated.\n\
+                In filter mode quantil only values inside the interval\n\
+                [<lower pq-quantil value>..<upper pq-quantil value>] are\n\
+                accepted for averaging. Values outside this interval are\n\
+                ignored."
 
 # define BlockNum 2       /* 1 input sequence + 1 output sequence */
 
@@ -109,9 +141,12 @@
 # define SSIGM        "sig-"  /* double */  // lower sigma limit factor
 # define SSIGP        "sig+"  /* double */  // upper sigma limit factor
 # define SMINS        "mins"  /* double */  // minimum sigma for filter 
-# define SPQ          "pq"    /* double */  // quantil value (0.5<=pq<1) 
+# define SLOQ         "loq"   /* double */  // lower quantil value (0=<loq<1) 
+# define SUPQ         "upq"   /* double */  // upper quantil value (0<upq<=1) 
+# define SPQ          "pq"    /* double */  // quantil value (0<=pq<=1) 
 
 const float eps=1e-32;
+const char percentchar='%';
 
 /***************************************************************************
 * Averaging Mode Translation Tables                                        *
@@ -216,6 +251,14 @@ PURPOSE
   Calculate average and variance for pixel (i_1,i_2). The type of average
   (mean, median) can be chosen. The routine takes into account dummy values.
 
+  In filter mode "mean", "median" the sigma values are used to calculated
+  the lower and upper limit values: lower value = sigma * lfac, upper value
+  = sigma * ufac.
+  In filter mode "quantil" all values outside the lower and upper quantil
+  value are rejected.
+  In average mode "mean", "median" and "quantil" the mean value, median 
+  value and pq-quantil values is calculated.
+
 ARGUMENTS
   const long dim[]        : input dimension dim = { 3, dim_1, dim_2, dim_3 }
   const float *frames (i) : input frames
@@ -223,15 +266,18 @@ ARGUMENTS
   const float *ddummy (i) : input ddummies, one value for each frame
   int i_1, int i_2 (i) : pixel indices 
                          (i_1 from 1 to dim_1, i_2 from 1 to dim_2)
-  (AMode) int amode   (i) : 1: mean, 2: median
-  (AMode) int fmode   (i) : 1: mean, 2: median
+  (AMode) int amode   (i) : 1: mean, 2: median, 3:quantil
+  (AMode) int fmode   (i) : 1: mean, 2: median, 3:quantil
   int vmode           (i) : 0: calculate variance using n
                             1: calculate variance using n-1
   int dovar           (i) : 0: do not return variance
                             1: return variance
-  float lfac          (i) : minimum sigma factor
-  float ufac          (i) : maximum sigma factor
+  float lfac          (i) : sigma factor for lower value 
+  float ufac          (i) : sigma factor for upper value 
   float mins          (i) : minimum sigma
+  float loq           (i) : lower quantil (only for fmode==quantil)
+  float upq           (i) : upper quantil (only for fmode==quantil)
+  float pq            (i) : quantil (only for amode==quantil)
 
   int *pstatus     (o) : return status 0==OK, otherwise error
 
@@ -247,23 +293,25 @@ AveValue average( const long dim[], const float *frames,
                   const float *dummy, const float *ddummy,
                   long i_1, long i_2, int amode, int fmode,
                   int vmode, int dovar, float lfac, float ufac,
-                  float mins, float pq, int *pstatus )
+                  float mins, float loq, float upq, float pq,
+                  int *pstatus )
 { AveValue ave;
   double *values=NULL;  // table of valid values (dim_3 elements) 
-  double value;
 
   const float *src;
   long k, pitch;
   double *dest;
   long nvalues=0, ivalues=0 ;    // number of valid values 
   double lower, upper;
-  double dlfac, dufac, dmins, dpq;
+  double dlfac, dufac, dmins, dloq, dupq, dpq;
 
   int status=Failed; 
   
   dlfac = (double) lfac;
   dufac = (double) ufac;
   dmins = (double) mins;
+  dloq   = (double) loq;
+  dupq   = (double) upq;
   dpq   = (double) pq;
 
   ave.val = 0.0;
@@ -282,6 +330,7 @@ AveValue average( const long dim[], const float *frames,
   nvalues = 0;
 
   for (k=0;k<dim[3];k++) {
+    double value;
     value = (double) *src;
 
     // UPDATE( *dest, value, dummy[k], ddummy[k] );
@@ -297,7 +346,7 @@ AveValue average( const long dim[], const float *frames,
   // 2) filter values
   if (fmode) {
     double sigma, val, var;
-    double upq, loq, hdif;
+    double upqval, loqval, hdif;
 
     // 2a) calculate average and deviation from average
     switch (fmode ) {
@@ -308,10 +357,10 @@ AveValue average( const long dim[], const float *frames,
         break;
 
       case Quantil:
-        upq = dquantil ( values, nvalues, 1.0-dpq );
-        loq = dquantil ( values, nvalues, dpq );
-        hdif = fabs((upq-loq)*0.5); // half distance beween quantils
-        val = (upq+loq)*0.5; // average of upper and lower quantil
+        upqval = dquantil ( values, nvalues, dupq );
+        loqval = dquantil ( values, nvalues, dloq );
+        hdif = fabs((upqval-loqval)*0.5); // half distance beween quantils
+        val = (upqval+loqval)*0.5; // average of upper and lower quantil
         sigma = hdif;
         break;
 
@@ -325,15 +374,24 @@ AveValue average( const long dim[], const float *frames,
     if (sigma<dmins) sigma=dmins;
 
     // 2b) calculate lower and upper limits
-    if (lfac<0.0)
-      lower = minimum ( values, nvalues ); // accept all below
-    else
-      lower = val - sigma * dlfac;
-
-    if (ufac<0.0)
-      upper = maximum ( values, nvalues ); // accept all above
-    else
-      upper = val + sigma * dufac;
+    switch (fmode ) {
+      case Quantil:
+        lower = val - sigma; // lower quantil value
+        upper = val + sigma; // upper quantil value
+        break;  
+      case Mean:
+      case Median:
+      default:
+        if (lfac<0.0)
+          lower = minimum ( values, nvalues ); // accept all below
+        else
+          lower = val - sigma * dlfac;
+    
+        if (ufac<0.0)
+          upper = maximum ( values, nvalues ); // accept all above
+        else
+          upper = val + sigma * dufac;
+    }
 
     // 2c) remove table values outside lower and upper limits
     ivalues = nvalues;
@@ -344,25 +402,21 @@ AveValue average( const long dim[], const float *frames,
   // 3) calculate average and variance
 
   switch (amode) {
-    double upq, loq, hdif;
     case Median: 
       ave.val = dmedian ( values, nvalues );
       if (dovar) 
-        ave.var = dmediance( values, nvalues, value );
+        ave.var = dmediance( values, nvalues, ave.val );
       break;
      case Quantil:
-        upq = dquantil ( values, nvalues, 1.0-dpq );
-        loq = dquantil ( values, nvalues, dpq );
-        hdif = fabs(upq-loq)*0.5; // half distance between quantils
-        ave.val = (upq+loq)*0.5; // average of upper and lower quantil
+        ave.val = dquantil ( values, nvalues, dpq );
         if (dovar)
-          ave.var = hdif*hdif;
+          ave.var = dquantilance(  values, nvalues, dpq, ave.val );
         break;
-    default: 
     case Mean: 
+    default: 
       ave.val = mean ( values, nvalues );
       if (dovar)
-        ave.var = variance( values, nvalues, value );
+        ave.var = variance( values, nvalues, ave.val );
   }
   ave.n   = nvalues;
   ave.i   = ivalues;
@@ -370,15 +424,13 @@ AveValue average( const long dim[], const float *frames,
   // 4) calculate variance of the average
   
   if (dovar) {
-    switch (amode) {
-      if (vmode==0) { // calculate with n values
-        if (nvalues>0)
-          ave.var /= nvalues;
-      }  else { // calculate with n-1 values
-        if (nvalues>1) 
-          ave.var /= nvalues-1;
-      }
-    }
+    if (vmode==0) { // calculate with n values
+       if (nvalues>0)
+         ave.var /= nvalues;
+     }  else { // calculate with n-1 values
+       if (nvalues>1) 
+         ave.var /= nvalues-1;
+     }
   }
 
   FREE(values);
@@ -684,10 +736,11 @@ void saxs_average_term (CmdBlk * pcb, long num[], ImgHeadBlk ihb[],
   IO_float SIGM;
   IO_float SIGP;
   IO_float MINS;
+  IO_float LOQ;
+  IO_float UPQ;
   IO_float PQ;
   int  amode;
   int  fmode;
-  float lfac=-1.0, ufac=-1.0;
 
   long  dim_1=0, dim_2=1, dim_3=1;
 
@@ -720,6 +773,12 @@ void saxs_average_term (CmdBlk * pcb, long num[], ImgHeadBlk ihb[],
   if (*pstatus!=Success)
     goto saxs_average_term_error;
   MINS = option_float ( SMINS, pcb, pcb->ib, num, 1, pstatus );
+  if (*pstatus!=Success)
+    goto saxs_average_term_error;
+  LOQ = option_float ( SLOQ, pcb, pcb->ib, num, 1, pstatus );
+  if (*pstatus!=Success)
+    goto saxs_average_term_error;
+  UPQ = option_float ( SUPQ, pcb, pcb->ib, num, 1, pstatus );
   if (*pstatus!=Success)
     goto saxs_average_term_error;
   PQ = option_float ( SPQ, pcb, pcb->ib, num, 1, pstatus );
@@ -735,6 +794,8 @@ void saxs_average_term (CmdBlk * pcb, long num[], ImgHeadBlk ihb[],
     printf("SIGM.V = %g\n",SIGM.V);
     printf("SIGP.V = %g\n",SIGP.V);
     printf("MINS.V = %g\n",MINS.V);
+    printf(" LOQ.V = %g\n",LOQ.V);
+    printf(" UPQ.V = %g\n",UPQ.V);
     printf("  PQ.V = %g\n",PQ.V);
   }
 
@@ -800,7 +861,7 @@ void saxs_average_term (CmdBlk * pcb, long num[], ImgHeadBlk ihb[],
           // value and variance (NM1.V==0: n values, NM1.V!=0: n-1 values)
           ave = average( dim, frames->data, frames->dummy, frames->ddummy,
                          i_1, i_2, amode, fmode, NM1.V, 1,
-                         SIGM.V, SIGP.V, MINS.V, PQ.V, &status );
+                         SIGM.V, SIGP.V, MINS.V, LOQ.V, UPQ.V, PQ.V, &status );
           if (status) {
             fprintf(ERROUT,"Error averaging frames at pixel %dx%d\n",i_1,i_2);
             goto saxs_average_term_error;
@@ -815,7 +876,7 @@ void saxs_average_term (CmdBlk * pcb, long num[], ImgHeadBlk ihb[],
           // value
           ave = average( dim, frames->data, frames->dummy, frames->ddummy,
                          i_1, i_2, amode, fmode, NM1.V, 0,
-                         SIGM.V, SIGP.V, MINS.V, PQ.V, &status );
+                         SIGM.V, SIGP.V, MINS.V, LOQ.V, UPQ.V, PQ.V, &status );
           if (status) {
             fprintf(ERROUT,"Error averaging frames at pixel %dx%d\n",i_1,i_2);
             goto saxs_average_term_error;
@@ -842,7 +903,8 @@ void saxs_average_term (CmdBlk * pcb, long num[], ImgHeadBlk ihb[],
     if (total>0)
       percent = 100.0 * (((double) ignored)/((double) total));
     else percent = 100.0;
-      printf("   %ld of %ld pixel values ignored (%3.1lg\%) \n",ignored,total,percent);
+    printf("   %ld of %ld pixel values ignored (%3.1lg%c) \n",
+      ignored,total,percent,percentchar);
   }
   printf("\n");
 
@@ -889,6 +951,8 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
   IO_flexp *pSIGM;
   IO_flexp *pSIGP;
   IO_flexp *pMINS;
+  IO_flexp *pLOQ;
+  IO_flexp *pUPQ;
   IO_flexp *pPQ;
 
   float ROff_1, RPs_1, UOff_1, UPs_1;
@@ -940,6 +1004,12 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
   pMINS = (IO_flexp*) option_parameter_search( SMINS, pcb, pstatus );
   if (*pstatus!=Success) return;
 
+  pLOQ = (IO_flexp*) option_parameter_search( SLOQ, pcb, pstatus );
+  if (*pstatus!=Success) return;
+
+  pUPQ = (IO_flexp*) option_parameter_search( SUPQ, pcb, pstatus );
+  if (*pstatus!=Success) return;
+
   pPQ = (IO_flexp*) option_parameter_search( SPQ, pcb, pstatus );
   if (*pstatus!=Success) return;
 
@@ -961,7 +1031,9 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
                        printf("sigma-               : %s\n", pSIGM->V);
                        printf("sigma+               : %s\n", pSIGP->V);
                        printf("minimum sigma        : %s\n", pMINS->V);
-                       printf("quantil value p      : %s\n", pPQ->V);
+                       printf("lower quantil loq    : %s\n", pLOQ->V);
+                       printf("upper quantil upq    : %s\n", pUPQ->V);
+                       printf("quantil (amode)      : %s\n", pPQ->V);
   printf("\n");
 
   if (pcb->TestBit) {
@@ -999,7 +1071,9 @@ int MAIN (int argc, char *argv[])
   option_define ( IO_tpflexp, SSIGM, "5", &cb ); // lower sigma factor
   option_define ( IO_tpflexp, SSIGP, "5", &cb ); // upper sigma factor
   option_define ( IO_tpflexp, SMINS, "0.5", &cb ); // minimum sigma per pixel
-  option_define ( IO_tpflexp, SPQ, "0.9", &cb ); // quantil value [1-p..p]
+  option_define ( IO_tpflexp, SLOQ, "0.1", &cb ); // lower quantil (fmode)
+  option_define ( IO_tpflexp, SUPQ, "0.9", &cb ); // upper quantil (fmode)
+  option_define ( IO_tpflexp, SPQ, "0.5", &cb ); // quantil (amode)
  
  /* Read options from argument list */
   ReadOptions( argv, &arg_no, &cb, ib, &status);
