@@ -76,9 +76,11 @@
   2005-08-04  PB V4.58 PrintImageHeaderBlock paramters updated
   2007-06-11  PB V4.59 ImageLoop parameters updated
   2012-01-31  PB V4.60 ImageLoop parameters updated
+  2013-01-18  PB V4.61 Dummy and output dimensions are skipped
+  2013-01-25  PB V4.62 Saxs2TiffFilenames according to ArgvBlocks
 
 ---*/
-# define Version  "saxs_tiff 4.60 2012-01-31, Peter Boesecke"
+# define Version  "saxs_tiff 4.62 2013-01-25, Peter Boesecke"
 
 # include <errno.h>
 # include <stdio.h>
@@ -281,18 +283,19 @@ The routine must be used for all images at once.
 ---------------------------------------------------------------------------*/
 void Saxs2TiffFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
                      int block_1st, int block_lst, int * pstatus)
-{ int blkno, first_block;
+{ int blkno, first_input, last_input;
   long image_1st, image_lst;
   char linebuffer[IO_len];
   char prefix[IO_len];
   long num[BlockNum];
   IO_line *pPrefix;
 
-  first_block = MAX2(1,block_1st);
+  first_input = MAX2(1,block_1st);
+  last_input  = block_lst;
 
   /* ---  File names and open modes --- */
 
-  for (blkno = first_block;blkno <= block_lst;blkno++) {
+  for (blkno = first_input;blkno <= last_input;blkno++) {
 
     /*--- Open mode ib[blkno].OpenMode of input files */
     if (!ib[blkno].OpenMode.I) {
@@ -363,7 +366,7 @@ void Saxs2TiffFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
 
   /* ---  Image numbers --- */
 
-  if (first_block==1) {
+  if (first_input==1) {
 
     /*--- Argument : number of first image */
     argv_long( pcb,"First image of input sequence 1", &ib[1].First, image_1st,
@@ -385,14 +388,9 @@ void Saxs2TiffFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
     if (*pstatus!=Success) return;
     if (pcb->TestBit) PrintImageHeaderBlock(&ihb[1]);
 
-    /* Defaults for output file */
-    if (!ib[0].Dummy.I) sprintf(ib[0].Dummy.V,"%g",ihb[1].Dummy.V);
-    if (!ib[0].Dim[1].I) sprintf(ib[0].Dim[1].V,"%d",ihb[1].Dim[1]);
-    if (!ib[0].Dim[2].I) sprintf(ib[0].Dim[2].V,"%d",ihb[1].Dim[2]);
-
   } /* first input file */
 
-  for (blkno = MAX2(2,first_block);blkno<=block_lst;blkno++) {
+  for (blkno = MAX2(2,first_input);blkno<=last_input;blkno++) {
     /*--- image numbers of all other images */
     (void) sprintf(linebuffer,"First image of input sequence %d",blkno);
     argv_long( pcb,linebuffer, &ib[blkno].First, ib[1].First.V, pstatus);
@@ -401,21 +399,37 @@ void Saxs2TiffFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
   } /* all other input files */
 
   if (block_1st <= 0) {
+
+    if (ib[0].First.I) num[0] = ib[0].First.V;
+    else num[0] = image_1st;
+
+    if ( (pcb->ImgBlkLen>1) && (ihb[1].I) ) {
+      NewImageHeader(pcb,ib,0,num,ihb,&(ihb[1]),NULL,pstatus);
+    } else {
+      NewImageHeader(pcb,ib,0,num,ihb,NULL,NULL,pstatus);
+    }
+    if (*pstatus!=Success) return;
+
     /*--- Argument : output dummy */
-    argv_flexp( pcb, "Output dummy", &ib[0].Dummy, ib[0].Dummy.V, pstatus);
+    sprintf(linebuffer,"Output dummy, e.g. %g",ihb[0].Dummy.V);
+    argv_flexp( pcb, linebuffer, &ib[0].Dummy, "", pstatus);
     if (*pstatus!=Success) return;
 
     /*--- Argument : output dimension 1 */
-    argv_lvexp(pcb,"Output dimension 1", &ib[0].Dim[1],ib[0].Dim[1].V,pstatus);
+    sprintf(linebuffer,"Output dimension 1, e.g. %ld",ihb[0].Dim[1]);
+    argv_lvexp(pcb,linebuffer,
+      &ib[0].Dim[1],ib[0].Dim[1].I?ib[0].Dim[1].V:"",pstatus);
     if (*pstatus!=Success) return;
 
     /*--- Argument : output dimension 2 */
-    argv_lvexp(pcb,"Output dimension 2", &ib[0].Dim[2],ib[0].Dim[2].V, pstatus);
+    sprintf(linebuffer,"Output dimension 2, e.g. %ld",ihb[0].Dim[2]);
+    argv_lvexp(pcb,linebuffer,
+      &ib[0].Dim[2],ib[0].Dim[2].I?ib[0].Dim[2].V:"", pstatus);
     if (*pstatus!=Success) return;
 
-    }
+  }
 
-  for (blkno = MAX2(1,first_block);blkno<=block_lst;blkno++) {
+  for (blkno = MAX2(1,first_input);blkno<=last_input;blkno++) {
     CloseImageFile( pcb, ib, blkno, pstatus) ;
     if (*pstatus!=Success) return;
   } /* close all input files */
